@@ -68,8 +68,10 @@ static char *do_iconv(GIConv conv, const char *word) {
 		return NULL;
 	char *out = out_buf;
 	size_t result = g_iconv(conv, &in, &len_in, &out, &len_out);
-	if (static_cast<size_t>(-1) == result)
+	if (static_cast<size_t>(-1) == result) {
+		free(out_buf);
 		return nullptr;
+        }
 	*out = '\0';
 	return out_buf;
 }
@@ -160,16 +162,18 @@ HunspellChecker::suggestWord(const char* const utf8Word, size_t len, size_t *nsu
 		return nullptr;
 
 	std::vector<std::string> sugMS = hunspell->suggest(out);
+	*nsug = sugMS.size();
 	g_free(out);
 	char **sug = g_new0 (char *, *nsug + 1);
 	if (sug) {
-		for (size_t i = 0, j = 0; i < *nsug; i++) {
+		size_t j = 0;
+		for (size_t i = 0; i < *nsug; i++) {
 			const char *in = sugMS[i].c_str();
 			out = do_iconv(m_translate_out, in);
 			if (out != NULL)
 				sug[j++] = out;
 		}
-		*nsug = sugMS.size();
+		*nsug = j;
 	} else
 		*nsug = 0;
 	return sug;
