@@ -1,6 +1,7 @@
 /* enchant
  * Copyright (C) 2022 Dimitrij Mijoski
  * Copyright (C) 2020 Sander van Geloven
+ * Copyright (C) 2024-2025 Reuben Thomas
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -69,26 +70,23 @@ static char** nuspell_dict_suggest(EnchantDict* me, const char* const word,
 	    UniquePtr(g_utf8_normalize(word, len, G_NORMALIZE_NFC), g_free);
 	auto suggestions = vector<string>();
 	dict->suggest(normalized_word.get(), suggestions);
-	if (empty(suggestions)) {
-		*out_n_suggs = 0;
-		return nullptr;
-	}
 	char** sug_list = g_new0(char*, size(suggestions) + 1);
-	transform(begin(suggestions), end(suggestions), sug_list,
-	          [](const string& sug) { return g_strdup(sug.c_str()); });
-	*out_n_suggs = size(suggestions);
+	if (sug_list) {
+		transform(begin(suggestions), end(suggestions), sug_list,
+			  [](const string& sug) { return g_strdup(sug.c_str()); });
+		*out_n_suggs = size(suggestions);
+	} else
+		*out_n_suggs = 0;
 	return sug_list;
 }
 // End EnchantDict functions
 
 // EnchantProvider functions
-static void nuspell_provider_dispose(EnchantProvider* me) { g_free(me); }
-
 static vector<filesystem::path>
 nuspell_get_dict_dirs(EnchantProvider *me)
 {
 	auto dirs = vector<filesystem::path>();
-	char *dir = enchant_get_user_dict_dir(me);
+	char *dir = enchant_provider_get_user_dict_dir(me);
 	dirs.push_back(std::filesystem::u8path(dir));
 	g_free(dir);
 	nuspell::append_default_dir_paths(dirs);
@@ -112,7 +110,9 @@ nuspell_provider_request_dict(EnchantProvider* me,
 		return nullptr;
 	}
 
-	EnchantDict* dict = g_new0(EnchantDict, 1);
+	EnchantDict* dict = enchant_dict_new();
+	if (dict == nullptr)
+		return nullptr;
 	dict->user_data = static_cast<void*>(dict_cpp.release());
 	dict->check = nuspell_dict_check;
 	dict->suggest = nuspell_dict_suggest;
@@ -124,7 +124,6 @@ static void nuspell_provider_dispose_dict(_GL_UNUSED EnchantProvider* me,
 {
 	auto dict_cpp = static_cast<nuspell::Dictionary*>(dict->user_data);
 	delete dict_cpp;
-	g_free(dict);
 }
 
 static int
@@ -155,10 +154,6 @@ nuspell_provider_list_dicts(EnchantProvider* me,
 	auto dirs = nuspell_get_dict_dirs(me);
 	auto dicts = vector<filesystem::path>();
 	nuspell::search_dirs_for_dicts(dirs, dicts);
-	if (empty(dicts)) {
-		*out_n_dicts = 0;
-		return nullptr;
-	}
 	for (auto& d : dicts)
 		d = d.stem();
 	sort(begin(dicts), end(dicts));
@@ -171,11 +166,14 @@ nuspell_provider_list_dicts(EnchantProvider* me,
 	dicts.erase(it, end(dicts));
 
 	char** dictionary_list = g_new0(char*, size(dicts) + 1);
-	transform(begin(dicts), end(dicts), dictionary_list,
-	          [](const filesystem::path& p) {
-		          return g_strdup(p.string().c_str());
-	          });
-	*out_n_dicts = size(dicts);
+	if (dictionary_list) {
+		transform(begin(dicts), end(dicts), dictionary_list,
+			  [](const filesystem::path& p) {
+				  return g_strdup(p.string().c_str());
+			  });
+		*out_n_dicts = size(dicts);
+	} else
+		*out_n_dicts = 0;
 	return dictionary_list;
 }
 
@@ -184,8 +182,7 @@ extern "C" EnchantProvider* init_enchant_provider(void);
 EnchantProvider *
 init_enchant_provider (void)
 {
-	EnchantProvider *provider = g_new0(EnchantProvider, 1);
-	provider->dispose = nuspell_provider_dispose;
+	EnchantProvider *provider = enchant_provider_new ();
 	provider->request_dict = nuspell_provider_request_dict;
 	provider->dispose_dict = nuspell_provider_dispose_dict;
 	provider->dictionary_exists = nuspell_provider_dictionary_exists;
