@@ -1,5 +1,6 @@
 /* Copyright (C) 2006 Barış Metin <baris@pardus.org.tr>
  * Copyright (C) 2007 Serkan Kaba <serkan_kaba@yahoo.com>
+ * Copyright (C) 2024-2025 Reuben Thomas
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -60,6 +61,8 @@ static bool zemberek_service_is_running ()
    return true;
 }
 
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Weffc++"
 class Zemberek
 {
 public:
@@ -73,6 +76,7 @@ private:
     DBusGConnection *connection;
     DBusGProxy *proxy;
 };
+#pragma GCC diagnostic pop
 
 Zemberek::Zemberek()
   : connection(nullptr), proxy(nullptr)
@@ -170,7 +174,9 @@ zemberek_provider_request_dict(EnchantProvider *me, const char *tag)
       {
         Zemberek* checker = new Zemberek();
 
-        EnchantDict* dict = enchant_broker_new_dict(me->owner);
+        EnchantDict* dict = enchant_dict_new();
+        if (dict == NULL)
+                return NULL;
         dict->user_data = (void *) checker;
         dict->check = zemberek_dict_check;
         dict->suggest = zemberek_dict_suggest;
@@ -214,9 +220,12 @@ zemberek_provider_list_dicts (EnchantProvider * me _GL_UNUSED,
     }
   else
     {
-        *out_n_dicts = 1;
+        *out_n_dicts = 0;
         char ** out_list = g_new0 (char *, 2);
-        out_list[0] = g_strdup ("tr");
+        if (out_list) {
+          out_list[0] = g_strdup ("tr");
+          *out_n_dicts = 1;
+        }
 
         return out_list;
     }
